﻿using System;
using System.Collections.ObjectModel;
using System.Drawing;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;

namespace ExtendingMapSuiteStyle
{
    public partial class TestForm : Form
    {
        private MapEngine mapEngine = new MapEngine();
        private Bitmap bitmap = null;

        public TestForm()
        {
            InitializeComponent();
        }

        private void TestForm_Load(object sender, EventArgs e)
        {
            // Set the full extent and the background color
            mapEngine.CurrentExtent = ExtentHelper.GetDrawingExtent(new RectangleShape(-180.0, 83.0, 180.0, -90.0), Map.Width, Map.Height);
            mapEngine.BackgroundFillBrush = new GeoSolidBrush(GeoColor.GeographicColors.DeepOcean);

            // Setup the countries layer
            ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(@"..\..\..\Dependencies\Countries02.shp",ShapeFileReadWriteMode.ReadOnly);
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
            worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            ShapeFileFeatureLayer worldCapitalsLayer = new ShapeFileFeatureLayer(@"..\..\..\Dependencies\WorldCapitals.shp", ShapeFileReadWriteMode.ReadOnly);            
            worldCapitalsLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            // Add the new layers to the MapEngine
            mapEngine.StaticLayers.Add("WorldCountries", worldLayer);
            mapEngine.StaticLayers.Add("WorldCapitals", worldCapitalsLayer);

            DrawImage();
        }

        private void btnSizedPoints_Click(object sender, EventArgs e)
        {
            ShapeFileFeatureLayer worldCapitalsLayer = (ShapeFileFeatureLayer)mapEngine.FindStaticFeatureLayer("WorldCapitals");

            SizedPointStyle sizedpointStyle = new SizedPointStyle(PointStyles.CreateSimpleCircleStyle(GeoColor.SimpleColors.Blue,1),"population",1000000);
            
            worldCapitalsLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Clear();
            worldCapitalsLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(sizedpointStyle);
            
            DrawImage();
        }

        private void btnTimePoints_Click(object sender, EventArgs e)
        {
            ShapeFileFeatureLayer worldCapitalsLayer = (ShapeFileFeatureLayer)mapEngine.FindStaticFeatureLayer("WorldCapitals");

            TimeBasedPointStyle timeBasedPointStyle = new TimeBasedPointStyle();
            timeBasedPointStyle.TimeZoneColumnName = "TimeZone";
            timeBasedPointStyle.DaytimePointStyle = PointStyles.CreateSimpleCircleStyle(GeoColor.StandardColors.Yellow, 8);
            timeBasedPointStyle.NighttimePointStyle = PointStyles.CreateSimpleCircleStyle(GeoColor.StandardColors.Gray, 8);
            
            worldCapitalsLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Clear();
            worldCapitalsLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(timeBasedPointStyle);
            
            DrawImage();
        }

        private void btnCachedValueStyle_Click(object sender, EventArgs e)
        {
            ShapeFileFeatureLayer worldCapitalsLayer = (ShapeFileFeatureLayer)mapEngine.FindStaticFeatureLayer("WorldCapitals");

            CachedValueStyle cachedValueStyle = new CachedValueStyle();
            cachedValueStyle.ColumnName = "pop_rank";            
            
            // Build the cache.  Normally this would come from SQl or someplace else
            worldCapitalsLayer.Open();
            Collection<Feature> features = worldCapitalsLayer.FeatureSource.GetAllFeatures(new string[] { "pop_rank" });
            foreach (Feature feature in features)
            {
                cachedValueStyle.ValuesCache.Add(feature.Id, feature.ColumnValues["pop_rank"].ToString());
            }
            worldCapitalsLayer.Close();

            cachedValueStyle.ValueItems.Add("1",new ValueItem("1", PointStyles.Capital1));
            cachedValueStyle.ValueItems.Add("2", new ValueItem("2", PointStyles.Capital2));
            cachedValueStyle.ValueItems.Add("3", new ValueItem("3", PointStyles.Capital3));

            worldCapitalsLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Clear();
            worldCapitalsLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(cachedValueStyle);

            DrawImage();
        }

        private void DrawImage()
        {
            if (bitmap != null) { bitmap.Dispose(); }
            bitmap = new Bitmap(Map.Width, Map.Height);
            mapEngine.OpenAllLayers();
            mapEngine.DrawStaticLayers(bitmap, GeographyUnit.DecimalDegree);
            mapEngine.CloseAllLayers();

            Map.Image = bitmap;
        }

        private void ToolBar_ButtonClick(object sender, ToolBarButtonClickEventArgs e)
        {
            switch (e.Button.Tag.ToString())
            {
                case "Zoom In":
                    mapEngine.CurrentExtent.ScaleDown(50);                    
                    break;
                case "Zoom Out":
                    mapEngine.CurrentExtent.ScaleUp(50);                    
                    break;
                case "Full Extent":
                    mapEngine.CurrentExtent = ExtentHelper.GetDrawingExtent(new RectangleShape(-180.0, 83.0, 180.0, -90.0), Map.Width, Map.Height);                    
                    break;
                case "Pan Left":
                    mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Left, 20);                    
                    break;
                case "Pan Right":
                    mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Right, 20);
                    break;
                case "Pan Up":
                    mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Up, 20);
                    break;
                case "Pan Down":
                    mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Down, 20);
                    break;
                default:
                    break;
            }
            DrawImage();
        }
    
        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}
