﻿Imports System
Imports System.Collections.Generic
Imports System.Collections.ObjectModel
Imports ThinkGeo.MapSuite.Core

Namespace VBExtendingMapSuiteStyle
    ' Mark the class Serializable so that it workes in SQL Server state 
    ' and in every serialization context 
    <Serializable()> _
    Public Class CachedValueStyle
        Inherits Style
        Private m_columnName As String
        Private m_valueItems As New Dictionary(Of String, ValueItem)()
        Private m_valuesCache As New Dictionary(Of String, String)()

        Public Sub New()
            Me.New(String.Empty, New Dictionary(Of String, ValueItem)())
        End Sub

        Public Sub New(ByVal columnName As String, ByVal valueItems As Dictionary(Of String, ValueItem))
            Me.m_columnName = columnName
            Me.m_valueItems = valueItems
        End Sub

        Public Property ColumnName() As String
            Get
                Return m_columnName
            End Get
            Set(ByVal value As String)
                m_columnName = value
            End Set
        End Property

        ' We use a dictionary now instead of a collection to make lookups faster. 
        Public ReadOnly Property ValueItems() As Dictionary(Of String, ValueItem)
            Get
                Return m_valueItems
            End Get
        End Property

        ' Here you can pre-load all of the values by feature Id. 
        ' The key is the Feature.Id and the value is the value to match the 
        ' value on. 
        Public ReadOnly Property ValuesCache() As Dictionary(Of String, String)
            Get
                Return m_valuesCache
            End Get
        End Property

        Protected Overloads Overrides Sub DrawCore(ByVal features As IEnumerable(Of Feature), ByVal canvas As GeoCanvas, ByVal labelsInThisLayer As Collection(Of SimpleCandidate), ByVal labelsInAllLayers As Collection(Of SimpleCandidate))
            For Each feature As Feature In features
                Dim fieldValue As String = String.Empty

                ' If we have items field value in the cache then let's use them! 
                ' If not then get the value from the feature's column values 
                If m_valuesCache.Count = 0 Then
                    fieldValue = feature.ColumnValues(m_columnName).Trim()
                Else
                    If Not ValuesCache.TryGetValue(feature.Id, fieldValue) Then
                        fieldValue = String.Empty
                    End If
                End If

                Dim valueItem As ValueItem = Nothing
                ' Check if the there is a value item in the dictionary that matches 
                ' our field value. 
                If m_valueItems.ContainsKey(fieldValue) Then
                    valueItem = m_valueItems(fieldValue)
                End If

                ' If we cannot find a matcing value item in the collection do not draw anything. 
                If valueItem IsNot Nothing Then
                    ' Call the draw on all of the default styles of the value item and also 
                    ' check if there are custom styles 
                    Dim tmpFeatures As Feature() = New Feature(0) {feature}
                    If valueItem.CustomStyles.Count = 0 Then
                        valueItem.DefaultAreaStyle.Draw(tmpFeatures, canvas, labelsInThisLayer, labelsInAllLayers)
                        valueItem.DefaultLineStyle.Draw(tmpFeatures, canvas, labelsInThisLayer, labelsInAllLayers)
                        valueItem.DefaultPointStyle.Draw(tmpFeatures, canvas, labelsInThisLayer, labelsInAllLayers)
                        valueItem.DefaultTextStyle.Draw(tmpFeatures, canvas, labelsInThisLayer, labelsInAllLayers)
                    Else
                        For Each style As Style In valueItem.CustomStyles
                            style.Draw(tmpFeatures, canvas, labelsInThisLayer, labelsInAllLayers)
                        Next
                    End If
                End If
            Next
        End Sub

        Protected Overloads Overrides Function GetRequiredColumnNamesCore() As Collection(Of String)
            Dim requiredFieldNames As New Collection(Of String)()

            ' If we have provided cached values then there is no need to fetch the 
            ' column from our feature source. 
            If m_valuesCache.Count = 0 Then
                requiredFieldNames.Add(m_columnName)
            End If

            For Each valueItem As ValueItem In m_valueItems.Values
                ' Check if we need any columns from custom styles 
                For Each style As Style In valueItem.CustomStyles
                    Dim tmpCollection As Collection(Of String) = style.GetRequiredColumnNames()

                    For Each name As String In tmpCollection
                        If Not requiredFieldNames.Contains(name) Then
                            requiredFieldNames.Add(name)
                        End If
                    Next
                Next

                ' Check if we need any columns from the DefaultTextStyle 
                Dim fieldsInTextStyle As Collection(Of String) = valueItem.DefaultTextStyle.GetRequiredColumnNames()
                For Each fieldName As String In fieldsInTextStyle
                    If Not requiredFieldNames.Contains(fieldName) Then
                        requiredFieldNames.Add(fieldName)
                    End If
                Next

                ' Check if we need any columns from the DefaultPointStyle 
                Dim fieldsInPointStyle As Collection(Of String) = valueItem.DefaultPointStyle.GetRequiredColumnNames()
                For Each fieldName As String In fieldsInPointStyle
                    If Not requiredFieldNames.Contains(fieldName) Then
                        requiredFieldNames.Add(fieldName)
                    End If
                Next

                ' Check if we need any columns from the DefaultLineStyle 
                Dim fieldsInLineStyle As Collection(Of String) = valueItem.DefaultLineStyle.GetRequiredColumnNames()
                For Each fieldName As String In fieldsInLineStyle
                    If Not requiredFieldNames.Contains(fieldName) Then
                        requiredFieldNames.Add(fieldName)
                    End If
                Next

                ' Check if we need any columns from the DefaultAreaStyle 
                Dim fieldsInAreaStyle As Collection(Of String) = valueItem.DefaultAreaStyle.GetRequiredColumnNames()
                For Each fieldName As String In fieldsInAreaStyle
                    If Not requiredFieldNames.Contains(fieldName) Then
                        requiredFieldNames.Add(fieldName)
                    End If
                Next
            Next

            Return requiredFieldNames
        End Function
    End Class
End Namespace