﻿Imports System
Imports System.Collections.ObjectModel
Imports System.Drawing
Imports System.Windows.Forms
Imports ThinkGeo.MapSuite.Core

Namespace VBExtendingMapSuiteStyle
    Partial Public Class TestForm
        Inherits Form
        Private mapEngine As New MapEngine()
        Private bitmap As Bitmap = Nothing

        Public Sub New()
            InitializeComponent()
        End Sub

        Private Sub TestForm_Load(ByVal sender As Object, ByVal e As EventArgs) Handles MyBase.Load
            ' Set the full extent and the background color 
            mapEngine.CurrentExtent = ExtentHelper.GetDrawingExtent(New RectangleShape(-180.0R, 83.0R, 180.0R, -90.0R), Map.Width, Map.Height)
            mapEngine.BackgroundFillBrush = New GeoSolidBrush(GeoColor.GeographicColors.DeepOcean)

            ' Setup the countries layer 
            Dim worldLayer As New ShapeFileFeatureLayer("..\..\..\Dependencies\Countries02.shp", ShapeFileReadWriteMode.[ReadOnly])
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1
            worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20

            Dim worldCapitalsLayer As New ShapeFileFeatureLayer("..\..\..\Dependencies\WorldCapitals.shp", ShapeFileReadWriteMode.[ReadOnly])
            worldCapitalsLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20

            ' Add the new layers to the MapEngine 
            mapEngine.StaticLayers.Add("WorldCountries", worldLayer)
            mapEngine.StaticLayers.Add("WorldCapitals", worldCapitalsLayer)

            DrawImage()
        End Sub

        Private Sub btnSizedPoints_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnSizedPoints.Click
            Dim worldCapitalsLayer As ShapeFileFeatureLayer = DirectCast(mapEngine.FindStaticFeatureLayer("WorldCapitals"), ShapeFileFeatureLayer)

            Dim sizedpointStyle As New SizedPointStyle(PointStyles.CreateSimpleCircleStyle(GeoColor.SimpleColors.Blue, 1), "population", 1000000)

            worldCapitalsLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Clear()
            worldCapitalsLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(sizedpointStyle)

            DrawImage()
        End Sub

        Private Sub btnTimePoints_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnTimePoints.Click
            Dim worldCapitalsLayer As ShapeFileFeatureLayer = DirectCast(mapEngine.FindStaticFeatureLayer("WorldCapitals"), ShapeFileFeatureLayer)

            Dim timeBasedPointStyle As New TimeBasedPointStyle()
            timeBasedPointStyle.TimeZoneColumnName = "TimeZone"
            timeBasedPointStyle.DaytimePointStyle = PointStyles.CreateSimpleCircleStyle(GeoColor.StandardColors.Yellow, 8)
            timeBasedPointStyle.NighttimePointStyle = PointStyles.CreateSimpleCircleStyle(GeoColor.StandardColors.Gray, 8)

            worldCapitalsLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Clear()
            worldCapitalsLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(timeBasedPointStyle)

            DrawImage()
        End Sub

        Private Sub btnCachedValueStyle_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnCachedValueStyle.Click
            Dim worldCapitalsLayer As ShapeFileFeatureLayer = DirectCast(mapEngine.FindStaticFeatureLayer("WorldCapitals"), ShapeFileFeatureLayer)

            Dim cachedValueStyle As New CachedValueStyle()
            cachedValueStyle.ColumnName = "pop_rank"

            ' Build the cache. Normally this would come from SQl or someplace else 
            worldCapitalsLayer.Open()
            Dim features As Collection(Of Feature) = worldCapitalsLayer.FeatureSource.GetAllFeatures(New String() {"pop_rank"})
            For Each feature As Feature In features
                cachedValueStyle.ValuesCache.Add(feature.Id, feature.ColumnValues("pop_rank").ToString())
            Next
            worldCapitalsLayer.Close()

            cachedValueStyle.ValueItems.Add("1", New ValueItem("1", PointStyles.Capital1))
            cachedValueStyle.ValueItems.Add("2", New ValueItem("2", PointStyles.Capital2))
            cachedValueStyle.ValueItems.Add("3", New ValueItem("3", PointStyles.Capital3))

            worldCapitalsLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Clear()
            worldCapitalsLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(cachedValueStyle)

            DrawImage()
        End Sub

        Private Sub DrawImage()
            If bitmap IsNot Nothing Then
                bitmap.Dispose()
            End If
            bitmap = New Bitmap(Map.Width, Map.Height)
            mapEngine.OpenAllLayers()
            mapEngine.DrawStaticLayers(bitmap, GeographyUnit.DecimalDegree)
            mapEngine.CloseAllLayers()

            Map.Image = bitmap
        End Sub

        Private Sub ToolBar_ButtonClick(ByVal sender As Object, ByVal e As ToolBarButtonClickEventArgs) Handles ToolBar.ButtonClick
            Select Case e.Button.Tag.ToString()
                Case "Zoom In"
                    mapEngine.CurrentExtent.ScaleDown(50)
                    Exit Select
                Case "Zoom Out"
                    mapEngine.CurrentExtent.ScaleUp(50)
                    Exit Select
                Case "Full Extent"
                    mapEngine.CurrentExtent = ExtentHelper.GetDrawingExtent(New RectangleShape(-180.0R, 83.0R, 180.0R, -90.0R), Map.Width, Map.Height)
                    Exit Select
                Case "Pan Left"
                    mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Left, 20)
                    Exit Select
                Case "Pan Right"
                    mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Right, 20)
                    Exit Select
                Case "Pan Up"
                    mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Up, 20)
                    Exit Select
                Case "Pan Down"
                    mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Down, 20)
                    Exit Select
                Case Else
                    Exit Select
            End Select
            DrawImage()
        End Sub

        Private Sub btnClose_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnClose.Click
            Me.Close()
        End Sub
    End Class
End Namespace
