﻿Imports System
Imports System.Collections.Generic
Imports System.Collections.ObjectModel
Imports ThinkGeo.MapSuite.Core

Namespace VBExtendingMapSuiteStyle
    ' Mark the class Serializable so that it workes in SQL Server state 
    ' and in every serialization context 
    <Serializable()> _
    Class TimeBasedPointStyle
        Inherits Style
        Private m_daytimePointStyle As PointStyle
        Private m_nighttimePointStyle As PointStyle
        Private m_timeZoneColumnName As String

        Public Sub New()
            Me.New(String.Empty, New PointStyle(), New PointStyle())
        End Sub

        Public Sub New(ByVal timeZoneColumnName As String, ByVal daytimePointStyle As PointStyle, ByVal nighttimePointStyle As PointStyle)
            Me.m_timeZoneColumnName = timeZoneColumnName
            Me.m_daytimePointStyle = daytimePointStyle
            Me.m_nighttimePointStyle = nighttimePointStyle
        End Sub

        Public Property DaytimePointStyle() As PointStyle
            Get
                Return m_daytimePointStyle
            End Get
            Set(ByVal value As PointStyle)
                m_daytimePointStyle = value
            End Set
        End Property

        Public Property NighttimePointStyle() As PointStyle
            Get
                Return m_nighttimePointStyle
            End Get
            Set(ByVal value As PointStyle)
                m_nighttimePointStyle = value
            End Set
        End Property

        Public Property TimeZoneColumnName() As String
            Get
                Return m_timeZoneColumnName
            End Get
            Set(ByVal value As String)
                m_timeZoneColumnName = value
            End Set
        End Property


        Protected Overloads Overrides Sub DrawCore(ByVal features As IEnumerable(Of Feature), ByVal canvas As GeoCanvas, ByVal labelsInThisLayer As Collection(Of SimpleCandidate), ByVal labelsInAllLayers As Collection(Of SimpleCandidate))
            For Each feature As Feature In features
                ' Here we are going to do the calculation to see what 
                ' time it is for each feature and draw the approperate style 
                Dim offsetToGmt As Single = Convert.ToSingle(feature.ColumnValues(m_timeZoneColumnName))
                Dim localTime As DateTime = DateTime.UtcNow.AddHours(offsetToGmt)
                Dim featureCollectioin As Collection(Of Feature) = New Collection(Of Feature)()
                featureCollectioin.Add(feature)
                If localTime.Hour >= 7 AndAlso localTime.Hour <= 19 Then
                    ' Daytime 
                    m_daytimePointStyle.Draw(featureCollectioin, canvas, labelsInThisLayer, labelsInAllLayers)
                Else
                    'Nighttime 
                    m_nighttimePointStyle.Draw(featureCollectioin, canvas, labelsInThisLayer, labelsInAllLayers)
                End If
            Next
        End Sub

        Protected Overloads Overrides Function GetRequiredColumnNamesCore() As Collection(Of String)
            Dim columns As New Collection(Of String)()

            ' Grab any columns that the daytime style may need. 
            Dim daytimeColumns As Collection(Of String) = m_daytimePointStyle.GetRequiredColumnNames()
            For Each column As String In daytimeColumns
                If Not columns.Contains(column) Then
                    columns.Add(column)
                End If
            Next

            ' Grab any columns that the nighttime style may need. 
            Dim nighttimeColumns As Collection(Of String) = m_nighttimePointStyle.GetRequiredColumnNames()
            For Each column As String In nighttimeColumns
                If Not columns.Contains(column) Then
                    columns.Add(column)
                End If
            Next

            ' Make sure we add the timezone column 
            If Not columns.Contains(m_timeZoneColumnName) Then
                columns.Add(m_timeZoneColumnName)
            End If

            Return columns
        End Function

        Protected Overloads Overrides Sub DrawSampleCore(ByVal canvas As GeoCanvas)
            ' Here is an example of why it is hard 
            ' to always draw a sample. In this case 
            ' we just draw the daytime stlye. 
            m_daytimePointStyle.DrawSample(canvas)
        End Sub
    End Class
End Namespace